import pygame
from pygame import Rect
from resource import get_image, get_sound
from utils import make_array
from level_io import Saveable, register_class
from animation import Animation

cell_size = (60, 120)
cell_depth = (20, 20)
wall_thickness = 10
floor_thickness = 10

cell_width, cell_height = cell_size
cell_hdepth, cell_vdepth = cell_depth

#cell_rect = Rect((0, 0), cell_size)
#wall_rect = Rect((0, 0),
#	(cell_depth[0] + wall_thickness, cell_depth[1] + cell_size[1]))
#floor_rect = Rect((0, 0),
#	(cell_depth[0] + cell_size[0], cell_depth[1] + floor_thickness))

class Level(object):
	#  num_rows   int
	#  num_cols   int
	#  backs      array[nrows, ncols] of Back
	#  gizmos     array[nrows, ncols] of Gizmo
	#  objects    array[nrows, ncols] of Object
	#  walls      array[nrows, ncols+1] of Wall
	#  floors     array[nrows+1, ncols] of Floor
	#  actors     list of Actor, including player
	#  player     Player
	
	def __init__(self, nrows, ncols):
		self.num_rows = nrows
		self.num_cols = ncols
		self.cells = make_array(None, nrows, ncols, 3)
		self.backs = self.cells[:,:,0]
		self.gizmos = self.cells[:,:,1]
		self.objects = self.cells[:,:,2]
		self.walls = make_array(None, nrows, ncols + 1)
		self.floors = make_array(None, nrows + 1, ncols)
		self.actors = []
		self.player = None
		self.filename = None
	
	def set_player(self, player):
		if self.player:
			self.actors.remove(self.player)
		self.player = player
		self.actors.append(player)

	def draw(self, surface):
		cw, ch = cell_size
		dw, dh = cell_depth
		nrows = self.num_rows
		ncols = self.num_cols
		#rows = xrange(self.num_rows)
		cols = xrange(self.num_cols)
		floor_rows = xrange(self.num_rows, -1, -1)
		wall_cols = xrange(self.num_cols + 1)
		cells = self.cells
		walls = self.walls
		floors = self.floors
		if self.player:
			prow, pcol = self.player.cell(self)
			if prow > 0:
				floor = self.floor_at(prow, pcol)
				if not floor or floor.passable():
					prow -= 1
		else:
			prow = pcol = None
		#print "Begin draw level"
		#print "Player is in cell", prow, pcol
		for row in floor_rows:
			if row < nrows:
				for col in wall_cols:
					wall = walls[row, col]
					if wall:
						r = Rect(0, 0, dw, ch + dh)
						r.topright = dw + col * cw, row * ch
						wall.draw(surface, r)
					if col < ncols:
						back, gizmo, obj = cells[row, col]
						r = Rect(col * cw, row * ch, cw + dw, ch + dh)
						if back:
							back.draw(surface, r)
						if gizmo:
							gizmo.draw(surface, r)
						if obj:
							obj.draw(surface, r)
					if row == prow and col == pcol:
						self.player.draw(surface, self)
			for col in cols:
				floor = floors[row, col]
				if floor:
					r = Rect(0, 0, cw + dw, dh)
					r.topright = dw + (col + 1) * cw, row * ch
					floor.draw(surface, r)
		#print "End draw level"
	
	def actor_pos_to_cell_coords(self, (x, y)):
		cw, ch = cell_size
		col, dx = divmod(x, cw)
		row, dy = divmod(y, ch)
		return (self.num_rows - row - 1, col), (dx, dy)
	
	def cell_coords_to_actor_pos(self, (row, col), (dx, dy)):
		cw, ch = cell_size
		x = col * cw + dx
		y = (self.num_rows - row - 1) * ch + dy
		return (x, y)

	def begin_frame(self):
		nrows = self.num_rows
		ncols = self.num_cols
		#backs = self.backs
		cells = self.cells
		walls = self.walls
		floors = self.floors
		for row in xrange(nrows + 1):
			for col in xrange(ncols + 1):
				coords = row, col
				if row < nrows and col < ncols:
					for item in cells[coords]:
						if item:
							item.begin_frame(self, coords)
					#back = backs[coords]
					#if back:
					#	back.begin_frame(self, coords)
				if row < nrows:
					wall = walls[coords]
					if wall:
						wall.begin_frame(self, coords)
				if col < ncols:
					floor = floors[coords]
					if floor:
						floor.begin_frame(self, coords)
		for actor in self.actors:
			actor.begin_frame(self)
	
	def wall_at(self, row, col):
		if 0 <= row < self.num_rows and 0 <= col <= self.num_cols:
			return self.walls[row, col]
		else:
			return None
		
	def floor_at(self, row, col):
		if 0 <= row <= self.num_rows and 0 <= col < self.num_cols:
			return self.floors[row, col]
		else:
			return None
	
	def gizmo_at(self, row, col):
		if 0 <= row < self.num_rows and 0 <= col < self.num_cols:
			return self.gizmos[row, col]
		else:
			return None
	
	def object_at(self, row, col):
		if 0 <= row < self.num_rows and 0 <= col < self.num_cols:
			return self.objects[row, col]
		else:
			return None
	
	def take_object_at(self, row, col):
		obj = self.object_at(row, col)
		if obj:
			self.objects[row, col] = None
		return obj
	
	def locate_player(self, coords):
		pos = self.cell_coords_to_actor_pos(coords, (30, 0))
		self.player.position = pos
	
	def update_tags(self):
		tags = {}
		for row in xrange(self.num_rows):
			for col in xrange(self.num_cols):
				item = self.gizmos[row, col]
				if item:
					tag = getattr(item, 'tag', None)
					if tag:
						tags[tag] = ((row, col), item)
		self.tags = tags
	
	def find_tag(self, tag):
		try:
			return self.tags[tag]
		except KeyError:
			return (0, 0), None
		

class Item(Saveable):
	#  image_name   string
	#  image        surface
	
	is_passable = False
	thumbnail = None
	editor_attrs = []
	border = 0
	
	def __init__(self, image, **kwds):
		Saveable.__init__(self, image = image, **kwds)
	
	def get_params(self):
		p = Saveable.get_params(self)
		p.image = self.image_name
		if self.border:
			p.border = self.border
		return p
	
	def set_params(self, p):
		self.image_name = p.image
		self.border = getattr(p, 'border', 0)
		self.image = get_image(p.image, border = self.border)
		#self.thumbnail = pygame.transform.rotozoom(self.image, 0, 0.5)

	def draw(self, surface, rect):
		image = self.image
		r = image.get_rect()
		r.topright = rect.topright
		surface.blit(image, r)
	
	def thumbnail_size(self):
		return self.image.get_size()
	
	def draw_thumbnail(self, surface, rect):
		r = self.image.get_rect()
		r.center = rect.center
		surface.blit(self.image, r)
	
	def begin_frame(self, level, coords):
		pass
	
	def passable(self):
		return self.is_passable
	
	def player_has_entered(self):
		pass


class Wall(Item):

	is_climbable = False
	
	def get_params(self):
		p = Item.get_params(self)
		p.passable = self.is_passable
		p.climbable = self.is_climbable
		return p
	
	def set_params(self, p):
		Item.set_params(self, p)
		cls = self.__class__
		self.is_passable = getattr(p, 'passable', cls.is_passable)
		self.is_climbable = getattr(p, 'climbable', cls.is_climbable)

	def draw(self, surface, rect):
		image = self.image
		r = image.get_rect()
		r.bottomright = rect.bottomright
		surface.blit(image, r)

	def climbable(self):
		return self.is_climbable
	
	def passed_by_player(self, player):
		pass


class Floor(Item):
	pass


class Back(Item):
	pass


class Gizmo(Item):

	height = None
	depth = None
	
	def get_params(self):
		p = Item.get_params(self)
		if self.height:
			p.height = self.height
		if self.depth:
			p.depth = self.depth
		return p

	def set_params(self, p):
		Item.set_params(self, p)
		self.height = getattr(p, 'height', None)
		self.depth = getattr(p, 'depth', None)

	def draw(self, surface, rect):
		image = self.image
		r = image.get_rect()
		h = self.height or r.height
		d = self.depth or r.height - h
		x, y = rect.topright
		if self.depth and not self.height:
			y += self.depth
		r.midtop = (x - (cell_width + d) // 2, y + cell_height - h)
		surface.blit(image, r)


class Object(Item):

	value = 0
	description = ""
	
	def __init__(self, image, description = "", **kwds):
		Item.__init__(self, image = image, description = description, **kwds)
	
	def get_params(self):
		p = Item.get_params(self)
		p.description = self.description
		return p
	
	def set_params(self, p):
		Item.set_params(self, p)
		self.description = getattr(p, 'description', "")

	def draw(self, surface, rect):
		#print "Object.draw in", rect
		image = self.image
		r = image.get_rect()
		x, y = rect.bottomright
		r.midbottom = (x - cell_width // 2, y - cell_vdepth // 2)
		surface.blit(image, r)


class BackDoor(Gizmo):
	is_static = False
	tag = ""
	link = ""
	open_sound = None
	editor_attrs = Gizmo.editor_attrs + [("Tag", 'tag'), ("Link", 'link')]
	
	def __init__(self, image, frames, border = 0, **kwds):
		Gizmo.__init__(self, image = image, frames = frames,
			border = border, **kwds)
	
	def get_params(self):
		p = Gizmo.get_params(self)
		p.frames = self.frames
		p.tag = self.tag
		p.link = self.link
		return p
	
	def set_params(self, p):
		Gizmo.set_params(self, p)
		self.frames = p.frames
		self.open_sound_name = getattr(p, 'open_sound', None)
		if self.open_sound_name:
			self.open_sound = get_sound(self.open_sound_name)
		self.tag = getattr(p, 'tag', "")
		self.link = getattr(p, 'link', "")
		self.anim = Animation(self.image_name, 1, self.frames, border = self.border)
	
	def begin_frame(self, level, coords):
		self.anim.begin_frame()
	
	def draw(self, surface, rect):
		x, y = rect.bottomright
		pos = (x - cell_width // 2, y - cell_vdepth)
		self.anim.draw(surface, 'midbottom', pos)
	
	def open(self):
		self.anim.resume_forward()
		if self.open_sound:
			self.open_sound.play()
	
	def close(self):
		self.anim.resume_backward()
	
	def set_open(self):
		self.anim.go_to_end()
	
	def set_closed(self):
		self.anim.go_to_beginning()
	
	def is_open(self):
		return self.anim.at_last_frame()

	def player_has_entered(self):
		self.anim.start_from_end()


class Battery(Object):

	kinds = ["red", "green", "blue"]

	def __init__(self, kind, image, description):
		Object.__init__(self, kind = kind, image = image,
			description = description)
	
	def get_params(self):
		p = Object.get_params(self)
		p.kind = self.kinds[self.kind]
		return p
	
	def set_params(self, p):
		Object.set_params(self, p)
		self.kind = self.kinds.index(p.kind)


class BeamWall(Wall):

	is_passable = True

	def __init__(self, color, image):
		Wall.__init__(self, color = color, image = image)
	
	def get_params(self):
		p = Wall.get_params(self)
		p.color = self.color
		return p
	
	def set_params(self, p):
		Wall.set_params(self, p)
		self.color = p.color
	
	def passed_by_player(self, player):
		if self.color <> player.cloak_tint():
			player.game.sound_alarm()


class Information(Object):

	is_static = False
	
	editor_attrs = Object.editor_attrs + [("Value", 'value', int), ("Description", 'description')]
	
	def __init__(self, image, value = 0, description = ""):
		Object.__init__(self, image = image, value = value,
			description = description)

	def get_params(self):
		p = Object.get_params(self)
		p.value = self.value
		return p

	def set_params(self, p):
		Object.set_params(self, p)
		self.value = p.value
		self.description = p.description


register_class(Wall)
register_class(Floor)
register_class(Back)
register_class(Gizmo)
register_class(Object)

register_class(BackDoor)
register_class(Battery)
register_class(BeamWall)
register_class(Information)
