from pygame import Color, Surface, SRCALPHA
from pygame.transform import rotozoom
from grid_view import GridView
from utils import frame_rect
from resource import get_image
from level import cell_size, cell_depth, wall_thickness, floor_thickness

tool_image_names = [
	"tool_select.png",
	"tool_add.png",
	"tool_erase.png",
]

class PaletteView(GridView):
	#  ncols   int   No. of columns
	#
	#  Abstract methods:
	#
	#    num_items()  -->  no. of items
	#    draw_item(surface, item_no, rect)
	#    click_item(item_no, event)
	#    item_is_selected(item_no)  -->  bool

	sel_color = (0, 0, 0)
	sel_width = 2

	def __init__(self, cell_size, nrows, ncols):
		GridView.__init__(self, cell_size, nrows, ncols)
		self.ncols = ncols
	
	def draw_cell(self, surface, row, col, rect):
		i = self.cell_to_item_no(row, col)
		if i is not None:
			self.draw_item(surface, i, rect)
			if self.item_is_selected(i):
				frame_rect(surface, self.sel_color, rect, self.sel_width)
				
	def click_cell(self, row, col, event):
		i = self.cell_to_item_no(row, col)
		if i is not None:
			self.click_item(i, event)
	
	def cell_to_item_no(self, row, col):
		i = row * self.num_cols() + col
		if 0 <= i < self.num_items():
			return i
		else:
			return None
	
	def num_rows(self):
		ncols = self.ncols
		return (self.num_items() + ncols - 1) // ncols
	
	def num_cols(self):
		return self.ncols


class ToolsView(PaletteView):

	bg_color = Color("burlywood")

	def __init__(self, editor):
		PaletteView.__init__(self, (32, 32), 1, len(tool_image_names))
		self.editor = editor
		self.images = map(get_image, tool_image_names)
	
	def num_items(self):
		return len(self.images)
	
	def draw_item(self, surface, i, rect):
		image = self.images[i]
		r = image.get_rect()
		r.center = rect.center
		surface.blit(image, r)
	
	def item_is_selected(self, i):
		return i == self.editor.current_tool
	
	def click_item(self, i, event):
		self.editor.current_tool = i


class ItemPaletteView(PaletteView):

	bg_color = Color("burlywood")
	cell_margin = 3
	scale = 2

	def __init__(self, editor, palette, item_size, nrows, ncols):
		w, h = item_size
		s = self.scale
		d = 2 * self.cell_margin
		PaletteView.__init__(self, (w//s + d, h//s + d), nrows, ncols)
		self.editor = editor
		self.palette = palette
		self.item_size = item_size
	
	def num_items(self):
		return len(self.palette)
	
	def draw_item(self, surface, i, rect):
		item = self.palette[i]
		x, y = rect.topleft
		d = self.cell_margin
		thumb = self.get_thumbnail(item)
		surface.blit(thumb, (x + d, y + d))

	def item_is_selected(self, i):
		item = self.palette[i]
		return item is self.editor.current_palette_item

	def click_item(self, i, event):
		self.editor.current_palette_item = self.palette[i]
	
	def get_thumbnail(self, item):
		thumb = item.thumbnail
		if not thumb:
			#print "ItemPaletteView: creating thumbnail for", item
			w, h = self.item_size
			buf = Surface((w, h), SRCALPHA, 32)
			item.draw(buf, buf.get_rect())
			thumb = rotozoom(buf, 0, 1.0 / self.scale)
			item.thumbnail = thumb
		return thumb


class WallPaletteView(ItemPaletteView):

	def __init__(self, editor, nrows, ncols):
		w = cell_depth[0] + wall_thickness
		h = cell_depth[1] + cell_size[1]
		ItemPaletteView.__init__(self, editor, editor.wall_palette,
			(w, h), nrows, ncols)


class FloorPaletteView(ItemPaletteView):

	def __init__(self, editor, nrows, ncols):
		w = cell_depth[0] + cell_size[0]
		h = cell_depth[1] + floor_thickness
		ItemPaletteView.__init__(self, editor, editor.floor_palette,
			(w, h), nrows, ncols)


class BackPaletteView(ItemPaletteView):

	def __init__(self, editor, nrows, ncols):
		ItemPaletteView.__init__(self, editor, editor.back_palette,
			cell_size, nrows, ncols)


class GizmoPaletteView(ItemPaletteView):

	def __init__(self, editor, nrows, ncols):
		ItemPaletteView.__init__(self, editor, editor.gizmo_palette,
			cell_size, nrows, ncols)


class ObjectPaletteView(ItemPaletteView):

	def __init__(self, editor, nrows, ncols):
		d = cell_size[0] + cell_depth[0]
		ItemPaletteView.__init__(self, editor, editor.object_palette,
			(d, d), nrows, ncols)

