import sys
from random import randint
from Numeric import add, maximum
from pygame import Rect
from pygame.locals import *
from widget import Widget
from root import RootWidget
from controls import Button
from resource import get_image, get_font, resource_path
from game import Game

heading_font = get_font(24, "VeraBd.ttf")
text_font = get_font(18)
default_start_level_name = "outside"

class Page(object):

	def __init__(self, heading, lines):
		self.heading = heading
		self.lines = lines
		width, height = heading_font.size(heading)
		for line in lines:
			w, h = text_font.size(line)
			width = max(width, w)
			height += h
		self.size = (width, height)
	
	def draw(self, surface, color, pos):
		x, y = pos
		buf = heading_font.render(self.heading, True, color)
		surface.blit(buf, (x, y))
		y += buf.get_rect().height
		for line in self.lines:
			buf = text_font.render(line, True, color)
			surface.blit(buf, (x, y))
			y += buf.get_rect().height
	

class Screen(Widget):

	bg_color = (155, 115, 11)
	border = 20


class TextScreen(Screen):

	fg_color = (0, 0, 0)

	def __init__(self, shell, filename):
		self.shell = shell
		text = open(resource_path("text", filename)).read()
		text_pages = text.split("PAGE")
		pages = []
		page_size = (0, 0)
		for text_page in text_pages:
			lines = text_page.strip().split("\n")
			page = Page(lines[0], lines[1:])
			pages.append(page)
			page_size = maximum(page_size, page.size)
		self.pages = pages
		bf = get_font(18, "VeraBd.ttf")
		b1 = Button("Prev Page", bf, action = self.prev_page)
		b2 = Button("Menu", bf, action = self.go_back)
		b3 = Button("Next Page", bf, action = self.next_page)
		b = self.border
		page_rect = Rect((b, b), page_size)
		gap = (0, 18)
		b1.rect.topleft = add(page_rect.bottomleft, gap)
		b2.rect.midtop = add(page_rect.midbottom, gap)
		b3.rect.topright = add(page_rect.bottomright, gap)
		Screen.__init__(self, Rect((0, 0), add(b3.rect.bottomright, (b, b))))
		self.add(b1)
		self.add(b2)
		self.add(b3)
		self.prev_button = b1
		self.next_button = b3
		self.set_current_page(0)
	
	def draw(self, surface):
		b = self.border
		self.pages[self.current_page].draw(surface, self.fg_color, (b, b))
	
	def at_first_page(self):
		return self.current_page == 0
	
	def at_last_page(self):
		return self.current_page == len(self.pages) - 1
	
	def set_current_page(self, n):
		self.current_page = n
		self.prev_button.enabled = not self.at_first_page()
		self.next_button.enabled = not self.at_last_page()

	def prev_page(self):
		if not self.at_first_page():
			self.set_current_page(self.current_page - 1)
	
	def next_page(self):
		if not self.at_last_page():
			self.set_current_page(self.current_page + 1)
	
	def go_back(self):
		self.shell.show_title()


class TitleScreen(Screen):

	def __init__(self, shell):
		self.image = get_image("title.png")
		font = get_font(24, "VeraBd.ttf")
		buttons = [
			Button("Intro", font, action = shell.do_intro),
			Button("Play", font, action = shell.do_play),
			Button("Quit", font, action = shell.do_quit),
		]
		ir = self.image.get_rect()
		b = self.border
		r = ir.inflate(2 * b, 2 * b)
		r.topleft = (0, 0)
		ir.center = r.center
		x, y = ir.midleft
		for b in buttons:
			b.rect.topleft = x, y
			y = b.rect.bottom + 5
		Widget.__init__(self, r)
		for b in buttons:
			self.add(b)
	
	def draw(self, surface):
		b = self.border
		surface.blit(self.image, (b, b))


class CaptureScreen(Widget):

	bg_color = (155, 115, 11)
	fg_color = (0, 0, 0)
	text_font = get_font(24, "VeraBd.ttf")

	def __init__(self, shell):
		self.shell = shell
		self.image = get_image("captured.png")
		r = self.image.get_rect()
		Widget.__init__(self, Rect((0, 0), add(r.size, (120, 120))))
	
	def draw(self, surface):
		surface.blit(self.image, (60, 40))
		if self.bribe:
			msg = "But for a bribe of $%s, the guard lets you go." % self.bribe
		else:
			msg = "You can't afford to bribe the guard. Game over."
		image = self.text_font.render(msg, True, self.fg_color)
		r = image.get_rect()
		r.midbottom = (self.rect.width // 2, self.rect.height - 30)
		surface.blit(image, r)
	
	def key_down(self, event):
		key = event.key
		if key == K_RETURN or key == K_KP_ENTER or key == K_ESCAPE:
			self.shell.do_resume()


class Shell(RootWidget):

	def __init__(self, screen, start_level = None, start_cell = None):
		RootWidget.__init__(self, screen)
		self.title_screen = TitleScreen(self)
		self.intro_screen = TextScreen(self, "intro.txt")
		self.capture_screen = CaptureScreen(self)
		self.game = None
		self.game_running = False
		self.current_screen = None
		self.show_screen(self.title_screen)
		if start_level:
			self.game = Game(self, start_level, start_cell)
			self.do_resume()
	
	def show_screen(self, screen):
		if self.current_screen is not screen:
			self.remove(self.current_screen)
			screen.rect.center = self.rect.center
			self.add(screen)
			self.current_screen = screen
	
	def show_title(self):
		self.show_screen(self.title_screen)

	def do_intro(self):
		self.show_screen(self.intro_screen)
	
	def do_play(self):
		if not self.game:
			self.game = Game(self, default_start_level_name)
		self.do_resume()
	
	def do_pause(self):
		self.game_running = False
		self.show_screen(self.title_screen)
	
	def do_resume(self):
		if self.game:
			self.show_screen(self.game)
			self.game_running = True
		else:
			self.show_screen(self.title_screen)
	
	def do_quit(self):
		sys.exit(0)

	def key_down(self, event):
		key = event.key
		if key == K_ESCAPE:
			self.do_pause()
		elif key == K_q and event.cmd:
			self.do_quit()
		else:
			if self.game_running:
				self.game.key_down(event)

	def key_up(self, event):
		if self.game_running:
			self.game.key_up(event)

	def begin_frame(self):
		if self.game_running:
			self.game.begin_frame()

	def player_captured(self):
		self.game_running = False
		player = self.game.player
		bribe = 50 * randint(1, 40)
		player.cash -= bribe
		if player.cash >= 0:
			self.show_capture_screen(bribe)
		else:
			self.game = None
			self.show_capture_screen(0)

	def show_capture_screen(self, bribe):
		screen = self.capture_screen
		screen.bribe = bribe
		self.show_screen(screen)
		screen.focus()
